@@ -24,13 +24,17 @@ from .decorators import pass_context, pass_obj, make_pass_decorator, \
      password_option, version_option, help_option
 
 # Types
-from .types import ParamType, File, Choice, STRING, INT, FLOAT, BOOL, UUID
+from .types import ParamType, File, Choice, IntRange, STRING, INT, FLOAT, \
+     BOOL, UUID
 
-# Helper functions
-from .helpers import prompt, confirm, get_terminal_size, echo, echo_via_pager
+# Utilities
+from .utils import echo
+
+# Terminal functions
+from .termui import prompt, confirm, get_terminal_size, echo_via_pager
 
 # Exceptions
-from .exceptions import UsageError, Abort
+from .exceptions import ClickException, UsageError, FileError, Abort
 
 # Formatting
 from .formatting import HelpFormatter, wrap_text
@@ -47,13 +51,17 @@ __all__ = [
     'version_option', 'help_option',
 
     # Types
-    'ParamType', 'File', 'Choice', 'STRING', 'INT', 'FLOAT', 'BOOL', 'UUID',
+    'ParamType', 'File', 'Choice', 'IntRange', 'STRING', 'INT', 'FLOAT',
+    'BOOL', 'UUID',
 
-    # Helper functions
-    'prompt', 'confirm', 'get_terminal_size', 'echo', 'echo_via_pager',
+    # Utilities
+    'echo',
+
+    # Terminal functions
+    'prompt', 'confirm', 'get_terminal_size', 'echo_via_pager',
 
     # Exceptions
-    'UsageError', 'Abort',
+    'ClickException', 'UsageError', 'FileError', 'Abort',
 
     # Formatting
     'HelpFormatter', 'wrap_text',
