@@ -3,10 +3,10 @@ import sys
 import codecs
 from itertools import chain
 
-from .types import convert_type, BOOL
-from .utils import make_str, make_default_short_help
-from .exceptions import UsageError, Abort
-from .helpers import prompt, confirm, echo
+from .types import convert_type, IntRange, BOOL
+from .utils import make_str, make_default_short_help, echo
+from .exceptions import ClickException, UsageError, Abort
+from .termui import prompt, confirm
 from .formatting import HelpFormatter
 from .parser import OptionParser
 
@@ -434,11 +434,9 @@ class Command(object):
             except (EOFError, KeyboardInterrupt):
                 echo(file=sys.stderr)
                 raise Abort()
-            except UsageError as e:
-                if e.ctx is not None:
-                    echo(e.ctx.get_usage() + '\n', file=sys.stderr)
-                echo('Error: %s' % e.message, file=sys.stderr)
-                sys.exit(2)
+            except ClickException as e:
+                e.show()
+                sys.exit(e.exit_code)
         except Abort:
             echo('Aborted!', file=sys.stderr)
             sys.exit(1)
@@ -787,6 +785,7 @@ class Option(Parameter):
                      multiple times and recorded.  This is similar to nargs
                      in how it works but supports arbitrary number of
                      arguments.
+    :param count: this flag makes an option increment an integer
     :param allow_from_autoenv: if this is enabled then the value of this
                                parameter will be pulled from an environment
                                variable in case a prefix is defined on the
@@ -798,8 +797,8 @@ class Option(Parameter):
     def __init__(self, param_decls=None, show_default=None,
                  prompt=False, confirmation_prompt=False,
                  hide_input=False, is_flag=None, flag_value=None,
-                 multiple=False, allow_from_autoenv=True, type=None,
-                 help=None, **attrs):
+                 multiple=False, count=False, allow_from_autoenv=True,
+                 type=None, help=None, **attrs):
         default_is_missing = attrs.get('default', _missing) is _missing
         Parameter.__init__(self, param_decls, type=type, **attrs)
         if prompt is True:
@@ -831,6 +830,10 @@ class Option(Parameter):
         self.flag_value = flag_value
         self.multiple = multiple
 
+        self.count = count
+        if count and type is None:
+            self.type = IntRange(min=0)
+
         if self.is_flag and isinstance(self.flag_value, bool) \
            and type is None:
             self.type = BOOL
@@ -851,6 +854,13 @@ class Option(Parameter):
                and self.prompt is not None:
                 raise TypeError('Hidden input does not work with boolean '
                                 'flag prompts.')
+            if self.count:
+                if self.multiple:
+                    raise TypeError('Options cannot be multiple and count '
+                                    'at the same time.')
+                elif self.is_flag:
+                    raise TypeError('Options cannot be count and flags at '
+                                    'the same time.')
 
     def _parse_decls(self, decls):
         opts = []
@@ -888,7 +898,12 @@ class Option(Parameter):
             'nargs': self.nargs,
         }
 
-        action = self.multiple and 'append' or 'store'
+        if self.multiple:
+            action = 'append'
+        elif self.count:
+            action = 'count'
+        else:
+            action = 'store'
 
         if self.is_flag:
             kwargs.pop('nargs', None)
