@@ -2,7 +2,7 @@ import inspect
 
 from functools import update_wrapper
 
-from .helpers import echo
+from .utils import echo
 
 
 def pass_context(f):
