@@ -1,9 +1,9 @@
 import sys
 import uuid
 
-from ._compat import open_stream, text_type
+from ._compat import open_stream, text_type, filename_to_ui, get_streerror
 from .exceptions import UsageError
-from .utils import safecall
+from .utils import safecall, LazyFile
 
 
 class ParamType(object):
@@ -86,6 +86,8 @@ class StringParamType(ParamType):
 class Choice(ParamType):
     """The choice type allows a value to checked against a fixed set of
     supported values.  All of these values have to be integers.
+
+    See :ref:`choice-opts` for an example.
     """
     name = 'choice'
 
@@ -118,6 +120,45 @@ class IntParamType(ParamType):
         return 'INT'
 
 
+class IntRange(IntParamType):
+    """A parameter that works similar to :data:`click.INT` but restricts
+    the value to fit into a range.  The default behavior is to fail if the
+    value falls outside the range, but it can also be silently clamped
+    between the two edges.
+
+    See :ref:`ranges` for an example.
+    """
+    name = 'integer range'
+
+    def __init__(self, min=None, max=None, clamp=False):
+        self.min = min
+        self.max = max
+        self.clamp = clamp
+
+    def convert(self, value, param, ctx):
+        rv = IntParamType.convert(self, value, param, ctx)
+        if self.clamp:
+            if self.min is not None and rv < self.min:
+                return self.min
+            if self.max is not None and rv > self.max:
+                return self.max
+        if self.min is not None and rv < self.min or \
+           self.max is not None and rv > self.max:
+            if self.min is None:
+                self.fail('%s is bigger than the maximum valid value '
+                          '%s.' % (rv, self.max), param, ctx)
+            elif self.max is None:
+                self.fail('%s is smaller than the minimum valid value '
+                          '%s.' % (rv, self.min), param, ctx)
+            else:
+                self.fail('%s is not in the valid range of %s to %s.'
+                          % (rv, self.min, self.max), param, ctx)
+        return rv
+
+    def __repr__(self):
+        return 'IntRange(%r, %r)' % (self.min, self.max)
+
+
 class BoolParamType(ParamType):
     name = 'boolean'
 
@@ -173,42 +214,57 @@ class File(ParamType):
     By default the file is opened for reading text data but it can also be
     opened in binary mode or for writing.  The encoding parameter can be used
     to force a specific encoding.
+
+    See :ref:`file-args` for more information.
     """
     name = 'filename'
 
-    def __init__(self, mode='r', encoding=None, errors='strict'):
+    def __init__(self, mode='r', encoding=None, errors='strict', lazy=None):
         self.mode = mode
         self.encoding = encoding
         self.errors = errors
+        self.lazy = lazy
+
+    def resolve_lazy_flag(self, value):
+        if self.lazy is not None:
+            return self.lazy
+        if value == '-':
+            return False
+        elif 'w' in self.mode:
+            return True
+        return False
 
     def convert(self, value, param, ctx):
         try:
             if hasattr(value, 'read') or hasattr(value, 'write'):
                 return value
-            f, was_opened = open_stream(value, self.mode, self.encoding,
-                                        self.errors)
+
+            lazy = self.resolve_lazy_flag(value)
+
+            if lazy:
+                f = LazyFile(value, self.mode, self.encoding, self.errors)
+                if ctx is not None:
+                    ctx.call_on_close(f.close_intelligently)
+                return f
+
+            f, should_close = open_stream(value, self.mode,
+                                          self.encoding, self.errors)
             # If a context is provided we automatically close the file
             # at the end of the context execution (or flush out).  If a
             # context does not exist it's the caller's responsibility to
             # properly close the file.  This for instance happens when the
             # type is used with prompts.
             if ctx is not None:
-                if was_opened:
+                if should_close:
                     ctx.call_on_close(safecall(f.close))
                 else:
                     ctx.call_on_close(safecall(f.flush))
             return f
         except (IOError, OSError) as e:
-            if isinstance(value, bytes):
-                value = value.decode(sys.getfilesystemencoding(), 'replace')
-            if hasattr(e, 'strerror'):
-                msg = e.strerror
-            else:
-                msg = str(e)
-            if isinstance(msg, bytes):
-                msg = msg.decode('utf-8', 'replace')
-            self.fail('Could not open file %s: %s' % (value, msg),
-                      param, ctx)
+            self.fail('Could not open file: %s: %s' % (
+                filename_to_ui(value),
+                get_streerror(e),
+            ), param, ctx)
 
 
 def convert_type(ty, default=None):
